@@ -1,7 +1,7 @@
 use std::{fmt::Write, num::FpCategory};
 
 use swc_atoms::js_word;
-use swc_common::{iter::IdentifyLast, util::take::Take, EqIgnoreSpan, Span, DUMMY_SP};
+use swc_common::{iter::IdentifyLast, util::take::Take, Span, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_utils::ident::IdentLike;
 
@@ -215,83 +215,8 @@ impl Pure<'_> {
         }
     }
 
-    pub(super) fn remove_duplicate_returns(&mut self, stmts: &mut Vec<Stmt>) {
-        fn drop_if_identical(last: &Stmt, check: &mut Stmt) -> bool {
-            if check.eq_ignore_span(last) {
-                check.take();
-                return true;
-            }
-
-            match check {
-                Stmt::Try(TryStmt {
-                    finalizer: Some(finalizer),
-                    ..
-                }) => {
-                    if let Some(check) = finalizer.stmts.last_mut() {
-                        if drop_if_identical(last, check) {
-                            return true;
-                        }
-                    }
-                }
-
-                Stmt::Try(TryStmt {
-                    handler: Some(CatchClause { body, .. }),
-                    finalizer: None,
-                    ..
-                }) => {
-                    if let Some(check) = body.stmts.last_mut() {
-                        if drop_if_identical(last, check) {
-                            return true;
-                        }
-                    }
-                }
-
-                Stmt::If(IfStmt { cons, alt, .. }) => {
-                    let mut changed = drop_if_identical(last, cons);
-                    if let Some(alt) = alt {
-                        if drop_if_identical(last, alt) {
-                            changed = true;
-                        }
-                    }
-
-                    return changed;
-                }
-
-                Stmt::Switch(SwitchStmt { cases, .. }) => {
-                    for case in cases {
-                        if let Some(check) = case.cons.last_mut() {
-                            if drop_if_identical(last, check) {
-                                return true;
-                            }
-                        }
-                    }
-                }
-
-                _ => {}
-            }
-
-            false
-        }
-
-        if stmts.is_empty() {
-            return;
-        }
-
-        let orig_len = stmts.len();
-        let (a, b) = stmts.split_at_mut(orig_len - 1);
-
-        if let Some(last @ Stmt::Return(..)) = b.last() {
-            if let Some(stmt_before_last) = a.last_mut() {
-                if drop_if_identical(last, stmt_before_last) {
-                    self.changed = true;
-                    report_change!("Dropped duplicate return");
-                }
-            }
-        }
-    }
-
     pub(super) fn remove_useless_return(&mut self, stmts: &mut Vec<Stmt>) {
-        if !self.options.dead_code && !self.options.reduce_vars {
+        if !self.options.dead_code {
             return;
         }
 
